@@ -1,8 +1,8 @@
 /* agent.c - Talking to gpg-agent.
-   Copyright (C) 2006, 2007, 2008 g10 Code GmbH
+   Copyright (C) 2006, 2007, 2008, 2015 g10 Code GmbH
 
    This file is part of Scute.
- 
+
    Scute is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
@@ -51,6 +51,7 @@
 
 #include "debug.h"
 #include "support.h"
+#include "sexp-parse.h"
 #include "cert.h"
 #include "agent.h"
 
@@ -65,7 +66,7 @@ static int agent_version_minor;
 
 
 /* Hack required for Windows.  */
-void 
+void
 gnupg_allow_set_foregound_window (pid_t pid)
 {
   if (!pid || pid == (pid_t)(-1))
@@ -78,305 +79,64 @@ gnupg_allow_set_foregound_window (pid_t pid)
 }
 
 
-#ifdef HAVE_W32_SYSTEM
-/* Helper function to build_w32_commandline. */
-static char *
-build_w32_commandline_copy (char *buffer, const char *string)
-{
-  char *p = buffer;
-  const char *s;
-
-  if (!*string) /* Empty string. */
-    p = stpcpy (p, "\"\"");
-  else if (strpbrk (string, " \t\n\v\f\""))
-    {
-      /* Need top do some kind of quoting.  */
-      p = stpcpy (p, "\"");
-      for (s=string; *s; s++)
-        {
-          *p++ = *s;
-          if (*s == '\"')
-            *p++ = *s;
-        }
-      *p++ = '\"';
-      *p = 0;
-    }
-  else
-    p = stpcpy (p, string);
-
-  return p;
-}
-
-
-/* Build a command line for use with W32's CreateProcess.  On success
-   CMDLINE gets the address of a newly allocated string.  */
-static gpg_error_t
-build_w32_commandline (const char *pgmname, const char * const *argv, 
-                       char **cmdline)
-{
-  int i, n;
-  const char *s;
-  char *buf, *p;
-
-  *cmdline = NULL;
-  n = 0;
-  s = pgmname;
-  n += strlen (s) + 1 + 2;  /* (1 space, 2 quoting */
-  for (; *s; s++)
-    if (*s == '\"')
-      n++;  /* Need to double inner quotes.  */
-  for (i=0; (s=argv[i]); i++)
-    {
-      n += strlen (s) + 1 + 2;  /* (1 space, 2 quoting */
-      for (; *s; s++)
-        if (*s == '\"')
-          n++;  /* Need to double inner quotes.  */
-    }
-  n++;
-
-  buf = p = malloc (n);
-  if (!buf)
-    return gpg_error_from_syserror ();
-
-  p = build_w32_commandline_copy (p, pgmname);
-  for (i=0; argv[i]; i++) 
-    {
-      *p++ = ' ';
-      p = build_w32_commandline_copy (p, argv[i]);
-    }
-
-  *cmdline= buf;
-  return 0;
-}
-
-
-/* Spawn a new process and immediatley detach from it.  The name of
-   the program to exec is PGMNAME and its arguments are in ARGV (the
-   programname is automatically passed as first argument).  An error
-   is returned if pgmname is not executable; to make this work it is
-   necessary to provide an absolute file name.  All standard file
-   descriptors are connected to /dev/null.  */
-static gpg_error_t
-spawn_process_detached (const char *pgmname, const char *argv[])
-{
-  gpg_error_t err;
-  SECURITY_ATTRIBUTES sec_attr;
-  PROCESS_INFORMATION pi = 
-    {
-      NULL,      /* Returns process handle.  */
-      0,         /* Returns primary thread handle.  */
-      0,         /* Returns pid.  */
-      0          /* Returns tid.  */
-    };
-  STARTUPINFO si;
-  int cr_flags;
-  char *cmdline;
-
-  if (access (pgmname, X_OK))
-    return gpg_error_from_syserror ();
-
-  /* Prepare security attributes.  */
-  memset (&sec_attr, 0, sizeof sec_attr );
-  sec_attr.nLength = sizeof sec_attr;
-  sec_attr.bInheritHandle = FALSE;
-  
-  /* Build the command line.  */
-  err = build_w32_commandline (pgmname, argv, &cmdline);
-  if (err)
-    return err; 
-
-  /* Start the process.  */
-  memset (&si, 0, sizeof si);
-  si.cb = sizeof (si);
-  si.dwFlags = STARTF_USESHOWWINDOW;
-  si.wShowWindow = SW_MINIMIZE;
-
-  cr_flags = (CREATE_DEFAULT_ERROR_MODE
-              | GetPriorityClass (GetCurrentProcess ())
-              | CREATE_NEW_PROCESS_GROUP
-              | DETACHED_PROCESS); 
-  DEBUG (DBG_INFO, "CreateProcess(detached), path=`%s' cmdline=`%s'\n",
-	 pgmname, cmdline);
-  if (!CreateProcess (pgmname,       /* Program to start.  */
-                      cmdline,       /* Command line arguments.  */
-                      &sec_attr,     /* Process security attributes.  */
-                      &sec_attr,     /* Thread security attributes.  */
-                      FALSE,         /* Inherit handles.  */
-                      cr_flags,      /* Creation flags.  */
-                      NULL,          /* Environment.  */
-                      NULL,          /* Use current drive/directory.  */
-                      &si,           /* Startup information. */
-                      &pi            /* Returns process information.  */
-                      ))
-    {
-      DEBUG (DBG_CRIT, "CreateProcess(detached) failed: %i\n",
-	     GetLastError ());
-      free (cmdline);
-      return gpg_error (GPG_ERR_GENERAL);
-    }
-  free (cmdline);
-  cmdline = NULL;
-
-  DEBUG (DBG_INFO, "CreateProcess(detached) ready: hProcess=%p hThread=%p"
-	 " dwProcessID=%d dwThreadId=%d\n", pi.hProcess, pi.hThread,
-	 (int) pi.dwProcessId, (int) pi.dwThreadId);
-
-  CloseHandle (pi.hThread); 
-
-  return 0;
-}
-#endif
-
 
 /* Establish a connection to a running GPG agent.  */
 static gpg_error_t
 agent_connect (assuan_context_t *ctx_r)
 {
-  /* If we ever failed to connect via a socket we will force the use
-     of the pipe based server for the lifetime of the process.  */
-  static int force_pipe_server = 0;
-
   gpg_error_t err = 0;
-  char *infostr;
-  char *ptr;
   assuan_context_t ctx = NULL;
+  char buffer[255];
+  FILE *p;
 
-  err = assuan_new (&ctx);
-  if (err)
-    return err;
-
- restart:
-
-  infostr = force_pipe_server ? NULL : getenv ("GPG_AGENT_INFO");
-  if (!infostr || !*infostr)
-    {
-      char *sockname;
-
-      /* First check whether we can connect at the standard
-         socket.  */
-      sockname = make_filename (default_homedir (), "S.gpg-agent", NULL);
-      if (! sockname)
-	return gpg_error_from_errno (errno);
-
-      err = assuan_socket_connect (ctx, sockname, 0, 0);
-      if (err)
-        {
-	  const char *agent_program;
-
-          /* With no success start a new server.  */
-	  DEBUG (DBG_INFO, "no running GPG agent at %s, starting one\n",
-		 sockname);
-
-          agent_program = get_gpg_agent_path ();
-
+  /* Use gpg-connect-agent to obtain the socket name
+   * directly from the agent itself. */
+  snprintf (buffer, sizeof buffer, "%s 'GETINFO socket_name' /bye",
+            get_gpg_connect_agent_path ());
 #ifdef HAVE_W32_SYSTEM
-          {
-            /* Under Windows we start the server in daemon mode.  This
-               is because the default is to use the standard socket
-               and thus there is no need for the GPG_AGENT_INFO
-               envvar.  This is possible as we don't have a real unix
-               domain socket but use a plain file and thus there is no
-               need to care about non-local file systems. */
-            const char *argv[3];
+  p = _popen (buffer, "r");
+#else
+  p = popen (buffer, "r");
+#endif
+  if (p)
+    {
+      int ret;
 
-            argv[0] = "--daemon";
-            argv[1] = "--use-standard-socket"; 
-            argv[2] = NULL;  
+      ret = fscanf (p, "D %254s\nOK\n", buffer);
+      if (ret == EOF)       /* I/O error? */
+        err = gpg_error_from_errno (errno);
+      else if (ret != 1)    /* Unexpected reply */
+        err = gpg_error (GPG_ERR_NO_AGENT);
 
-            err = spawn_process_detached (agent_program, argv);
-            if (err)
-              DEBUG (DBG_CRIT, "failed to start agent `%s': %s\n",
-		     agent_program, gpg_strerror (err));
-            else
-              {
-                /* Give the agent some time to prepare itself. */
-                Sleep (3 * 1000);
-                /* Now try again to connect the agent.  */
-                err = assuan_socket_connect (ctx_r, sockname, 0, 0);
-              }
-          }
-#else /*!HAVE_W32_SYSTEM*/
-          {
-            const char *pgmname;
-            const char *argv[3];
-            int no_close_list[3];
-            int i;
-
-            if ( !(pgmname = strrchr (agent_program, '/')))
-              pgmname = agent_program;
-            else
-              pgmname++;
-            
-            argv[0] = pgmname;
-            argv[1] = "--server";
-            argv[2] = NULL;
-            
-            i=0;
-            no_close_list[i++] = assuan_fd_from_posix_fd (fileno (stderr));
-            no_close_list[i] = -1;
-            
-            /* Connect to the agent and perform initial handshaking. */
-            err = assuan_pipe_connect (ctx, agent_program, argv,
-				       no_close_list, NULL, NULL, 0);
-          }
-#endif /*!HAVE_W32_SYSTEM*/
-        }
-      free (sockname);
+      pclose (p);
     }
   else
+    err = gpg_error_from_errno (errno);
+
+  /* Then connect to the socket we got. */
+  if (!err)
     {
-      int pid;
-      int protocol_version;
-
-      infostr = strdup (infostr);
-      if (!infostr)
-	return gpg_error_from_errno (errno);
-
-      if (!(ptr = strchr (infostr, PATHSEP_C)) || ptr == infostr)
-	{
-	  DEBUG (DBG_CRIT, "malformed GPG_AGENT_INFO environment variable");
-	  free (infostr);
-	  force_pipe_server = 1;
-	  goto restart;
-	}
-
-      *(ptr++) = 0;
-      pid = atoi (ptr);
-      while (*ptr && *ptr != PATHSEP_C)
-	ptr++;
-      protocol_version = *ptr ? atoi (ptr + 1) : 0;
-      if (protocol_version != 1)
-	{
-	  DEBUG (DBG_CRIT, "GPG agent protocol version '%d' not supported",
-		 protocol_version);
-	  free (infostr);
-	  force_pipe_server = 1;
-	  goto restart;
-	}
-      
-      err = assuan_socket_connect (ctx, infostr, pid, 0);
-      free (infostr);
-      if (err)
-	{
-	  DEBUG (DBG_CRIT, "cannot connect to GPG agent: %s", gpg_strerror (err));
-	  force_pipe_server = 1;
-	  goto restart;
-	}
+      err = assuan_new (&ctx);
+      if (!err)
+        {
+          err = assuan_socket_connect (ctx, buffer, 0, 0);
+          if (!err)
+            {
+              *ctx_r = ctx;
+              if (_scute_debug_flags & DBG_ASSUAN)
+                assuan_set_log_stream (*ctx_r, _scute_debug_stream);
+            }
+          else
+            assuan_release (ctx);
+        }
     }
 
+  /* We do not try any harder. If gpg-connect-agent somehow failed
+   * to give us a suitable socket, we probably cannot do better. */
   if (err)
-    {
-      assuan_release (ctx);
-      DEBUG (DBG_CRIT, "cannot connect to GPG agent: %s", gpg_strerror (err));
-      return gpg_error (GPG_ERR_NO_AGENT);
-    }
+    DEBUG (DBG_CRIT, "cannot connect to GPG agent: %s", gpg_strerror (err));
 
-  if (_scute_debug_flags & DBG_ASSUAN)
-    assuan_set_log_stream (*ctx_r, _scute_debug_stream);
-
-  *ctx_r = ctx;
-
-  return 0;
+  return err;
 }
 
 
@@ -400,7 +160,7 @@ default_inq_cb (void *opaque, const char *line)
 
 
 /* Send a simple command to the agent.  */
-static gpg_error_t 
+static gpg_error_t
 agent_simple_cmd (assuan_context_t ctx, const char *fmt, ...)
 {
   gpg_error_t err;
@@ -421,7 +181,7 @@ agent_simple_cmd (assuan_context_t ctx, const char *fmt, ...)
     DEBUG (DBG_CRIT, "gpg-agent command '%s' failed: %s", optstr,
 	   gpg_strerror (err));
   free (optstr);
-      
+
   return err;
 }
 
@@ -432,7 +192,9 @@ read_version_cb (void *opaque, const void *buffer, size_t length)
 {
   char version[20];
   const char *s;
-  
+
+  (void) opaque;
+
   if (length > sizeof (version) -1)
     length = sizeof (version) - 1;
   strncpy (version, buffer, length);
@@ -444,7 +206,7 @@ read_version_cb (void *opaque, const void *buffer, size_t length)
 
   return 0;
 }
-  
+
 
 /* Configure the GPG agent at connection CTX.  */
 static gpg_error_t
@@ -538,15 +300,11 @@ agent_configure (assuan_context_t ctx)
   if (dft_pinentry_user_data)
     err = agent_simple_cmd (ctx, "OPTION pinentry_user_data=%s",
 	                    dft_pinentry_user_data);
-  if (gpg_err_code (err) == GPG_ERR_UNKNOWN_OPTION)
-    err = 0;
-  else if (err)
+  if (err && gpg_err_code (err) != GPG_ERR_UNKNOWN_OPTION)
     return err;
 
   err = agent_simple_cmd (ctx, "OPTION allow-pinentry-notify");
-  if (gpg_err_code (err) == GPG_ERR_UNKNOWN_OPTION)
-    err = 0;
-  else if (err)
+  if (err && gpg_err_code (err) != GPG_ERR_UNKNOWN_OPTION)
     return err;
 
   err = assuan_transact (ctx, "GETINFO version",
@@ -615,7 +373,7 @@ unescape_status_string (const unsigned char *src)
   while (*src)
     {
       if (*src == '%' && src[1] && src[2])
-        { 
+        {
           src++;
           *dst = xtoi_2 (src);
           if (*dst == '\0')
@@ -631,13 +389,13 @@ unescape_status_string (const unsigned char *src)
       else
         *(dst++) = *(src++);
     }
-  *dst = 0; 
+  *dst = 0;
 
   return buffer;
 }
 
 
-/* Take a 20 byte hexencoded string and put it into the the provided
+/* Take a 20 byte hexencoded string and put it into the provided
    20 byte buffer FPR in binary format.  Returns true if successful,
    and false otherwise.  */
 static int
@@ -652,7 +410,6 @@ unhexify_fpr (const char *hexstr, unsigned char *fpr)
   if ((*src && !spacep (src)) || (cnt != 40))
     return 0;
 
-  cnt /= 2;
   for (src = hexstr, cnt = 0; *src && !spacep (src); src += 2, cnt++)
     fpr[cnt] = xtoi_2 (src);
 
@@ -848,6 +605,30 @@ learn_status_cb (void *opaque, const char *line)
 	    }
 	}
     }
+  else if (keywordlen == 6 && !memcmp (keyword, "EXTCAP", keywordlen))
+    {
+      char *p, *p2, *buf;
+      int abool;
+
+      buf = p = unescape_status_string (line);
+      if (buf)
+        {
+          for (p = strtok (buf, " "); p; p = strtok (NULL, " "))
+            {
+              p2 = strchr (p, '=');
+              if (p2)
+                {
+                  *p2++ = 0;
+                  abool = (*p2 == '1');
+                  if (!strcmp (p, "gc"))
+                    parm->rng_available = abool;
+                  /* We're currently not interested in the
+                   * other capabilities. */
+                }
+            }
+          free (buf);
+        }
+    }
   return 0;
 }
 
@@ -894,7 +675,7 @@ geteventcounter_status_cb (void *opaque, const char *line)
           last_count = count;
         }
     }
-  
+
   return 0;
 }
 
@@ -954,8 +735,11 @@ scute_agent_check_status (void)
 }
 
 
-/* Enough space to hold a 2048 bit RSA signature in an S-expression.  */
-#define MAX_SIGNATURE_LEN 350
+/* We only support RSA signatures up to 4096 bits.  */
+#define MAX_SIGNATURE_BITS 4096
+
+/* Enough space to hold a 4096 bit RSA signature in an S-expression.  */
+#define MAX_SIGNATURE_LEN 640	/* FIXME: magic value */
 
 struct signature
 {
@@ -980,14 +764,142 @@ pksign_cb (void *opaque, const void *buffer, size_t length)
   return 0;
 }
 
+/* Parse the result of an pksign operation which is a s-expression in
+   normal form that looks like (7:sig-val(3:rsa(1:s<LENGTH>:<DATA>))).
+   The raw result is stored in RESULT of size *LEN, and *LEN is
+   adjusted to the actual size.  */
+static gpg_error_t
+pksign_parse_result (const struct signature *sig,
+                     unsigned char *result, unsigned int *len)
+{
+  gpg_error_t err;
+  const unsigned char *s = sig->data;
+  size_t n;
+  int depth;
 
-#define SIG_PREFIX   "(7:sig-val(3:rsa(1:s128:"
-#define SIG_PREFIX_2 "(7:sig-val(3:rsa(1:s256:"
-#define SIG_PREFIX_LEN (sizeof (SIG_PREFIX) - 1)
-#define SIG_POSTFIX ")))"
-#define SIG_POSTFIX_LEN (sizeof (SIG_POSTFIX) - 1)
-#define SIG_LEN 128
-#define SIG_LEN_2 256
+  if (*s++ != '(')
+    gpg_error (GPG_ERR_INV_SEXP);
+
+  n = snext (&s);
+  if (! n)
+    return gpg_error (GPG_ERR_INV_SEXP);
+  if (! smatch (&s, n, "sig-val"))
+    return gpg_error (GPG_ERR_UNKNOWN_SEXP);
+
+  if (*s++ != '(')
+    gpg_error (GPG_ERR_UNKNOWN_SEXP);
+
+  n = snext (&s);
+  if (! n)
+    return gpg_error (GPG_ERR_INV_SEXP);
+  if (! smatch (&s, n, "rsa"))
+    return gpg_error (GPG_ERR_UNKNOWN_SEXP);
+
+  if (*s++ != '(')
+    gpg_error (GPG_ERR_UNKNOWN_SEXP);
+
+  n = snext (&s);
+  if (! n)
+    return gpg_error (GPG_ERR_INV_SEXP);
+  if (! smatch (&s, n, "s"))
+    return gpg_error (GPG_ERR_UNKNOWN_SEXP);
+
+  n = snext (&s);
+  if (! n)
+    return gpg_error (GPG_ERR_INV_SEXP);
+
+  /* Remove nul byte prepended by gpg-agent. */
+  if (!*s && n > 1)
+    {
+      n -= 1;
+      s += 1;
+    }
+
+  if (*len < (unsigned int) n)
+    return gpg_error (GPG_ERR_INV_LENGTH);
+
+  *len = (unsigned int) n;
+  memcpy (result, s, n);
+  s += n;
+
+  depth = 3;
+  err = sskip (&s, &depth);
+  if (err)
+    return err;
+  if (s - sig->data != sig->len || depth != 0)
+    return gpg_error (GPG_ERR_INV_SEXP);
+
+  return 0;
+}
+
+/* Decodes the hash DATA of size LEN (if necessary).  Returns a
+   pointer to the raw hash data in R_DATA, the size in R_LEN, and the
+   name of the hash function in R_HASH.
+
+   Prior to TLSv1.2, the hash function was the concatenation of MD5
+   and SHA1 applied to the data respectively, and no encoding was
+   applied.  From TLSv1.2 on, the hash value is prefixed with an hash
+   identifier and encoded using ASN1.
+
+   FIXME: Reference.  */
+static gpg_error_t
+decode_hash (const unsigned char *data, int len,
+             const unsigned char **r_data, size_t *r_len,
+             const char **r_hash)
+{
+  static unsigned char rmd160_prefix[15] = /* Object ID is 1.3.36.3.2.1 */
+    { 0x30, 0x21, 0x30, 0x09, 0x06, 0x05, 0x2b, 0x24, 0x03,
+      0x02, 0x01, 0x05, 0x00, 0x04, 0x14  };
+  static unsigned char sha1_prefix[15] =   /* (1.3.14.3.2.26) */
+    { 0x30, 0x21, 0x30, 0x09, 0x06, 0x05, 0x2b, 0x0e, 0x03,
+      0x02, 0x1a, 0x05, 0x00, 0x04, 0x14  };
+  static unsigned char sha224_prefix[19] = /* (2.16.840.1.101.3.4.2.4) */
+    { 0x30, 0x2D, 0x30, 0x0d, 0x06, 0x09, 0x60, 0x86, 0x48,
+      0x01, 0x65, 0x03, 0x04, 0x02, 0x04, 0x05, 0x00, 0x04,
+      0x1C  };
+  static unsigned char sha256_prefix[19] = /* (2.16.840.1.101.3.4.2.1) */
+    { 0x30, 0x31, 0x30, 0x0d, 0x06, 0x09, 0x60, 0x86,
+      0x48, 0x01, 0x65, 0x03, 0x04, 0x02, 0x01, 0x05,
+      0x00, 0x04, 0x20  };
+  static unsigned char sha384_prefix[19] = /* (2.16.840.1.101.3.4.2.2) */
+    { 0x30, 0x41, 0x30, 0x0d, 0x06, 0x09, 0x60, 0x86,
+      0x48, 0x01, 0x65, 0x03, 0x04, 0x02, 0x02, 0x05,
+      0x00, 0x04, 0x30  };
+  static unsigned char sha512_prefix[19] = /* (2.16.840.1.101.3.4.2.3) */
+    { 0x30, 0x51, 0x30, 0x0d, 0x06, 0x09, 0x60, 0x86,
+      0x48, 0x01, 0x65, 0x03, 0x04, 0x02, 0x03, 0x05,
+      0x00, 0x04, 0x40  };
+
+#define HANDLE(hash,hashlen)                                            \
+  if (len == sizeof hash ## _prefix + (hashlen)                         \
+      && !memcmp (data, hash ## _prefix, sizeof hash ## _prefix))       \
+    {                                                                   \
+      *r_data = data + sizeof hash ## _prefix;                          \
+      *r_len = hashlen;                                                 \
+      *r_hash = #hash;                                                  \
+    }
+
+  if (len == 36)
+    {
+      /* Prior to TLSv1.2, a combination of MD5 and SHA1 was used.  */
+      *r_data = data;
+      *r_len = 36;
+      *r_hash = "tls-md5sha1";
+    }
+      /* TLSv1.2 encodes the hash value using ASN1.  */
+  else HANDLE (sha1,   20)
+  else HANDLE (rmd160, 20)
+  else HANDLE (sha224, 28)
+  else HANDLE (sha256, 32)
+  else HANDLE (sha384, 48)
+  else HANDLE (sha512, 64)
+    else
+      return gpg_error (GPG_ERR_INV_ARG);
+
+#undef HANDLE
+
+  return 0;
+}
 
 /* Call the agent to learn about a smartcard.  */
 gpg_error_t
@@ -996,7 +908,10 @@ scute_agent_sign (char *grip, unsigned char *data, int len,
 {
   char cmd[150];
   gpg_error_t err;
-#define MAX_DATA_LEN 36
+  const char *hash;
+  const unsigned char *raw_data;
+  size_t raw_len;
+#define MAX_DATA_LEN	64	/* Size of an SHA512 sum.  */
   unsigned char pretty_data[2 * MAX_DATA_LEN + 1];
   int i;
   struct signature sig;
@@ -1006,17 +921,17 @@ scute_agent_sign (char *grip, unsigned char *data, int len,
   if (sig_len == NULL)
     return gpg_error (GPG_ERR_INV_ARG);
 
+  err = decode_hash (data, len, &raw_data, &raw_len, &hash);
+  if (err)
+    return err;
+
   if (sig_result == NULL)
     {
-      /* FIXME:  We return the largest supported size - is that correct?  */
-      *sig_len = SIG_LEN_2;
+      *sig_len = raw_len;
       return 0;
     }
 
-  if (len > MAX_DATA_LEN)
-    return gpg_error (GPG_ERR_INV_ARG);
-
-  if (grip == NULL || sig_result == NULL || *sig_len < SIG_LEN)
+  if (grip == NULL || sig_result == NULL)
     return gpg_error (GPG_ERR_INV_ARG);
 
   snprintf (cmd, sizeof (cmd), "SIGKEY %s", grip);
@@ -1025,11 +940,11 @@ scute_agent_sign (char *grip, unsigned char *data, int len,
   if (err)
     return err;
 
-  for (i = 0; i < len; i++)
-    snprintf (&pretty_data[2 * i], 3, "%02X", data[i]);
-  pretty_data[2 * len] = '\0';
+  for (i = 0; i < raw_len; i++)
+    snprintf (&pretty_data[2 * i], 3, "%02X", raw_data[i]);
+  pretty_data[2 * raw_len] = '\0';
 
-  snprintf (cmd, sizeof (cmd), "SETHASH --hash=tls-md5sha1 %s", pretty_data);
+  snprintf (cmd, sizeof (cmd), "SETHASH --hash=%s %s", hash, pretty_data);
   err = assuan_transact (agent_ctx, cmd, NULL, NULL, default_inq_cb,
 			 NULL, NULL, NULL);
   if (err)
@@ -1040,32 +955,8 @@ scute_agent_sign (char *grip, unsigned char *data, int len,
   if (err)
     return err;
 
-  /* FIXME: we need a real parser to cope with all kind of S-expressions.  */
-  if (sig.len == SIG_PREFIX_LEN + SIG_LEN_2 + SIG_POSTFIX_LEN)
-    {
-      if (memcmp (sig.data, SIG_PREFIX_2, SIG_PREFIX_LEN))
-        return gpg_error (GPG_ERR_BAD_SIGNATURE);
-      if (memcmp (sig.data + sig.len - SIG_POSTFIX_LEN,
-                  SIG_POSTFIX, SIG_POSTFIX_LEN))
-        return gpg_error (GPG_ERR_BAD_SIGNATURE);
-      memcpy (sig_result, sig.data + SIG_PREFIX_LEN, SIG_LEN_2);
-      *sig_len = SIG_LEN_2;
-    }
-  else
-    {
-      if (sig.len != SIG_PREFIX_LEN + SIG_LEN + SIG_POSTFIX_LEN)
-        return gpg_error (GPG_ERR_BAD_SIGNATURE);
-      if (memcmp (sig.data, SIG_PREFIX, SIG_PREFIX_LEN))
-        return gpg_error (GPG_ERR_BAD_SIGNATURE);
-      if (memcmp (sig.data + sig.len - SIG_POSTFIX_LEN,
-                  SIG_POSTFIX, SIG_POSTFIX_LEN))
-        return gpg_error (GPG_ERR_BAD_SIGNATURE);
-      memcpy (sig_result, sig.data + SIG_PREFIX_LEN, SIG_LEN);
-      *sig_len = SIG_LEN;
-    }
-  
-  
-  return 0;
+  err = pksign_parse_result (&sig, sig_result, sig_len);
+  return err;
 }
 
 
@@ -1104,7 +995,6 @@ gpg_error_t
 get_cert_data_cb (void *opaque, const void *data, size_t data_len)
 {
   struct get_cert_s *cert_s = opaque;
-  gpg_error_t err;
   int needed_size;
 
   needed_size = cert_s->cert_der_len + data_len;
@@ -1153,7 +1043,7 @@ scute_agent_get_cert (int no, struct cert *cert)
   err = assuan_transact (agent_ctx, cmd, get_cert_data_cb, &cert_s,
 			 NULL, NULL, NULL, NULL);
   /* Just to be safe... */
-  if (!err && cert_s.cert_der_len <= 16)
+  if (!err && (cert_s.cert_der_len <= 16 || cert_s.cert_der[0] != 0x30))
     {
       DEBUG (DBG_INFO, "bad card certificate rejected");
       err = gpg_error (GPG_ERR_BAD_CERT);
@@ -1174,6 +1064,42 @@ scute_agent_get_cert (int no, struct cert *cert)
   return 0;
 }
 
+struct random_request
+{
+    unsigned char *buffer;
+    size_t len;
+};
+
+gpg_error_t
+get_challenge_data_cb (void *opaque, const void *line, size_t len)
+{
+  struct random_request *request = opaque;
+
+  if (len != request->len)
+    return gpg_error (GPG_ERR_INV_LENGTH);
+
+  memcpy (request->buffer, line, len);
+
+  return 0;
+}
+
+gpg_error_t
+scute_agent_get_random (unsigned char *data, size_t len)
+{
+    char command[16];
+    gpg_error_t err;
+    struct random_request request;
+
+    snprintf (command, sizeof(command), "SCD RANDOM %zu", len);
+
+    request.buffer = data;
+    request.len = len;
+    err = assuan_transact (agent_ctx, command, get_challenge_data_cb,
+                           &request, NULL, NULL, NULL, NULL);
+
+    return err;
+}
+
 
 void
 scute_agent_finalize (void)
