@@ -42,9 +42,19 @@
 #include "cryptoki.h"
 #include "support.h"
 #include "cert.h"
+#include "debug.h"
 
 
+/* Parsing certificates is problematic, as there is no reliable
+   standard.  However, we must parse at least the subject field (see
+   below).  So, disabling cert parsing does currently not disable it
+   completely.  The purpose of the symbol is to document which parts
+   of the code are mandatory, and which are optional.  */
+#define CERT_PARSING 1
+
 
+#if CERT_PARSING || 1
+
 #define atoi_1(p)   (*(p) - '0' )
 #define atoi_2(p)   ((atoi_1(p) * 10) + atoi_1((p)+1))
 #define atoi_4(p)   ((atoi_2(p) * 100) + atoi_2((p)+2))
@@ -76,7 +86,11 @@ time_to_ck_date (time_t *atime, CK_DATE *ckdate)
   if (!(broken_time.tm_year >= 0 && broken_time.tm_year <= 8099
 	&& broken_time.tm_mon >= 0 && broken_time.tm_mon <= 11
 	&& broken_time.tm_mday >= 1 && broken_time.tm_mday <= 31))
-    return false;
+    { 
+      DEBUG (DBG_INFO, "unrepresentable time %i-%i-%i",
+	     broken_time.tm_year, broken_time.tm_mon, broken_time.tm_mday);
+      return false;
+    }
 
 #define LAST_DIGIT(d) (((d) % 10) + '0')
   nr = broken_time.tm_year + 1900;
@@ -111,7 +125,10 @@ asn1_get_len (unsigned char **asn1, int *asn1_len, int *rlen)
   int result = 0;
 
   if (len < 1)
-    return gpg_error (GPG_ERR_GENERAL);
+    {
+      DEBUG (DBG_INFO, "unexpected end of certificate");
+      return gpg_error (GPG_ERR_GENERAL);
+    }
 
   if (*ptr & 0x80)
     {
@@ -123,8 +140,16 @@ asn1_get_len (unsigned char **asn1, int *asn1_len, int *rlen)
     cnt = 1;
 
   /* We only support a limited number of length bytes.  */
-  if (cnt > 2 || len < cnt)
-    return gpg_error (GPG_ERR_GENERAL);
+  if (cnt > 2)
+    {
+      DEBUG (DBG_INFO, "unsupported length field");
+      return gpg_error (GPG_ERR_GENERAL);
+    }
+  if (len < cnt)
+    {
+      DEBUG (DBG_INFO, "unexpected end of certificate");
+      return gpg_error (GPG_ERR_GENERAL);
+    }
 
   while (cnt--)
     {
@@ -166,8 +191,16 @@ asn1_get_element (unsigned char *cert, int cert_len,
   for (i = 0; i < path_size; i++)
     {
       prev_certp = certp;
-      if (cert_left < 1 || *certp != path[i].tag)
-	return gpg_error (GPG_ERR_GENERAL);
+      if (cert_left < 1)
+	{
+	  DEBUG (DBG_INFO, "unexpected end of certificate");
+	  return gpg_error (GPG_ERR_GENERAL);
+	}
+      if (*certp != path[i].tag)
+	{
+	  DEBUG (DBG_INFO, "wrong element in lookup path");
+	  return gpg_error (GPG_ERR_GENERAL);
+	}
       certp++;
       cert_left--;
       err = asn1_get_len (&certp, &cert_left, &len);
@@ -176,7 +209,10 @@ asn1_get_element (unsigned char *cert, int cert_len,
       if (!path[i].enter)
 	{
 	  if (cert_left < len)
-	    return gpg_error (GPG_ERR_GENERAL);
+	    {
+	      DEBUG (DBG_INFO, "unexpected end of certificate");
+	      return gpg_error (GPG_ERR_GENERAL);
+	    }
 	  certp += len;
 	  cert_left -= len;
 	}
@@ -186,8 +222,16 @@ asn1_get_element (unsigned char *cert, int cert_len,
 	     bit string.  */
 	  if (path[i].tag == '\x03')
 	    {
-	      if (cert_left < 1 || *certp != '\x00')
-		return gpg_error (GPG_ERR_GENERAL);
+	      if (cert_left < 1)
+		{
+		  DEBUG (DBG_INFO, "unexpected end of certificate");
+		  return gpg_error (GPG_ERR_GENERAL);
+		}
+	      if (*certp != '\x00')
+		{
+		  DEBUG (DBG_INFO, "expected binary encapsulation missing");
+		  return gpg_error (GPG_ERR_GENERAL);
+		}
 	      certp++;
 	      cert_left--;
 	    }
@@ -271,7 +315,10 @@ asn1_get_modulus (unsigned char *cert, int cert_len,
     return err;
 
   if (*sub_len < 1)
-    return gpg_error (GPG_ERR_GENERAL);
+    {
+      DEBUG (DBG_INFO, "modulus too short");
+      return gpg_error (GPG_ERR_GENERAL);
+    }
 
   (*sub_start)++;
   (*sub_len)--;
@@ -313,7 +360,10 @@ asn1_get_public_exp (unsigned char *cert, int cert_len,
     return err;
 
   if (*sub_len < 1)
-    return gpg_error (GPG_ERR_GENERAL);
+    {
+      DEBUG (DBG_INFO, "public exponent too short");
+      return gpg_error (GPG_ERR_GENERAL);
+    }
 
   (*sub_start)++;
   (*sub_len)--;
@@ -330,6 +380,7 @@ asn1_get_public_exp (unsigned char *cert, int cert_len,
 
   return 0;
 }
+#endif
 
 
 static gpg_error_t
@@ -341,7 +392,10 @@ attr_one (CK_ATTRIBUTE_PTR attr, CK_ULONG *attr_count,
   attr[i].ulValueLen = size;
   attr[i].pValue = malloc (size);
   if (attr[i].pValue == NULL)
-    return gpg_error (GPG_ERR_ENOMEM);
+    {
+      DEBUG (DBG_CRIT, "out of memory");
+      return gpg_error (GPG_ERR_ENOMEM);
+    }
   memcpy (attr[i].pValue, val, size);
   (*attr_count)++;
   return 0;
@@ -373,7 +427,7 @@ gpg_error_t
 scute_attr_cert (struct cert *cert,
 		 CK_ATTRIBUTE_PTR *attrp, CK_ULONG *attr_countp)
 {
-  CK_RV err;
+  CK_RV err = 0;
   CK_ATTRIBUTE_PTR attr;
   CK_ULONG attr_count;
 
@@ -399,22 +453,46 @@ scute_attr_cert (struct cert *cert,
   CK_DATE obj_end_date;
   CK_ULONG obj_java_midp_sec_domain = 0;
 
+#if CERT_PARSING || 1
+  /* See below.  */
   err = asn1_get_subject (cert->cert_der, cert->cert_der_len,
 			  &subject_start, &subject_len);
-  if (!err)
-    err = asn1_get_issuer (cert->cert_der, cert->cert_der_len,
-			   &issuer_start, &issuer_len);
-  if (!err)
-    err = asn1_get_serial (cert->cert_der, cert->cert_der_len,
-			   &serial_start, &serial_len);
   if (err)
-    return err;
+    {
+      DEBUG (DBG_INFO, "rejecting certificate: could not get subject: %s",
+	     gpg_strerror (err));
+      return err;
+    }
+#endif
+#if CERT_PARSING
+  err = asn1_get_issuer (cert->cert_der, cert->cert_der_len,
+			 &issuer_start, &issuer_len);
+  if (err)
+    {
+      DEBUG (DBG_INFO, "rejecting certificate: could not get issuer: %s",
+	     gpg_strerror (err));
+      return err;
+    }
+
+  err = asn1_get_serial (cert->cert_der, cert->cert_der_len,
+			 &serial_start, &serial_len);
+  if (err)
+    {
+      DEBUG (DBG_INFO, "rejecting certificate: could not get serial: %s",
+	     gpg_strerror (err));
+      return err;
+    }
+#endif
+
 
 #define NR_ATTR_CERT 20
   attr = malloc (sizeof (CK_ATTRIBUTE) * NR_ATTR_CERT);
   attr_count = 0;
   if (!attr)
-    return gpg_error (GPG_ERR_ENOMEM);
+    {
+      DEBUG (DBG_INFO, "out of memory");
+      return gpg_error (GPG_ERR_ENOMEM);
+    }
 
 #define one_attr_ext(type, val, size)					\
   if (!err)								\
@@ -438,28 +516,48 @@ scute_attr_cert (struct cert *cert,
   /* FIXME: Calculate check_value.  */
   one_attr (CKA_CHECK_VALUE, obj_check_value);
 
+#if 0
   if (time_to_ck_date (&cert->timestamp, &obj_start_date))
     {
       one_attr (CKA_START_DATE, obj_start_date);
     }
-  else
-    {
-      empty_attr (CKA_START_DATE);
-    }
 
   if (time_to_ck_date (&cert->expires, &obj_end_date))
     {
       one_attr (CKA_END_DATE, obj_end_date);
     }
-  else
-    {
-      empty_attr (CKA_END_DATE);
-    }
+#else
+  /* For now, we disable these fields.  We can parse them from the
+     certificate just as the other data.  However, we would like to
+     avoid parsing the certificates at all, let's see how much
+     functionality we really need in the PKCS#11 token first.  */
+  empty_attr (CKA_START_DATE);
+  empty_attr (CKA_END_DATE);
+#endif
 
+  /* Note: This attribute is mandatory.  Without it, Firefox client
+     authentication won't work.  */
+#if CERT_PARSING || 1
   one_attr_ext (CKA_SUBJECT, subject_start, subject_len);
+#endif
+
+#if 0
+  /* If we get the info directly from the card, we don't have a
+     fingerprint, and parsing the subject key identifier is quite a
+     mouth full.  Let's try a different approach for now.  */
   one_attr_ext (CKA_ID, cert->fpr, 40);
+#else
+  {
+    char certptr[40];
+    snprintf (certptr, DIM (certptr), "%p", cert);
+    one_attr_ext (CKA_ID, certptr, strlen (certptr));
+  }
+#endif
+
+#if CERT_PARSING
   one_attr_ext (CKA_ISSUER, issuer_start, issuer_len);
   one_attr_ext (CKA_SERIAL_NUMBER, serial_start, serial_len);
+#endif
   one_attr_ext (CKA_VALUE, cert->cert_der, cert->cert_der_len);
   
   empty_attr (CKA_URL);
@@ -470,12 +568,14 @@ scute_attr_cert (struct cert *cert,
 
   if (err)
     {
+      DEBUG (DBG_INFO, "could not build certificate object: %s",
+	     gpg_strerror (err));
       scute_attr_free (attr, attr_count);
       return err;
     }
 
   /* FIXME: Not completely safe.  */
-  assert (NR_ATTR_CERT == attr_count);
+  assert (NR_ATTR_CERT >= attr_count);
 
   *attrp = attr;
   *attr_countp = attr_count;
@@ -487,7 +587,7 @@ gpg_error_t
 scute_attr_prv (struct cert *cert, CK_ATTRIBUTE_PTR *attrp,
 		CK_ULONG *attr_countp)
 {
-  CK_RV err;
+  CK_RV err = 0;
   CK_ATTRIBUTE_PTR attr;
   CK_ULONG attr_count;
 
@@ -524,22 +624,41 @@ scute_attr_prv (struct cert *cert, CK_ATTRIBUTE_PTR *attrp,
   CK_BBOOL obj_wrap_with_trusted = CK_FALSE;
   CK_BBOOL obj_always_authenticate = CK_FALSE;
 
+#if CERT_PARSING
   err = asn1_get_subject (cert->cert_der, cert->cert_der_len,
 			  &subject_start, &subject_len);
-  if (!err)
-    err = asn1_get_modulus (cert->cert_der, cert->cert_der_len,
-			    &modulus_start, &modulus_len);
-  if (!err)
-    err = asn1_get_public_exp (cert->cert_der, cert->cert_der_len,
-			       &public_exp_start, &public_exp_len);
   if (err)
-    return err;
+    {
+      DEBUG (DBG_INFO, "rejecting certificate: could not get subject: %s",
+	     gpg_strerror (err));
+      return err;
+    }
+  err = asn1_get_modulus (cert->cert_der, cert->cert_der_len,
+			  &modulus_start, &modulus_len);
+  if (err)
+    {
+      DEBUG (DBG_INFO, "rejecting certificate: could not get modulus: %s",
+	     gpg_strerror (err));
+      return err;
+    }
+  err = asn1_get_public_exp (cert->cert_der, cert->cert_der_len,
+			     &public_exp_start, &public_exp_len);
+  if (err)
+    {
+      DEBUG (DBG_INFO, "rejecting certificate: could not get public exp: %s",
+	     gpg_strerror (err));
+      return err;
+    }
+#endif
 
 #define NR_ATTR_PRV 27
   attr = malloc (sizeof (CK_ATTRIBUTE) * NR_ATTR_PRV);
   attr_count = 0;
   if (!attr)
-    return gpg_error (GPG_ERR_ENOMEM);
+    {
+      DEBUG (DBG_INFO, "out of core");
+      return gpg_error (GPG_ERR_ENOMEM);
+    }
 
 #undef one_attr_ext
 #define one_attr_ext(type, val, size)					\
@@ -561,32 +680,54 @@ scute_attr_prv (struct cert *cert, CK_ATTRIBUTE_PTR *attrp,
   one_attr (CKA_LABEL, obj_label);
 
   one_attr (CKA_KEY_TYPE, obj_key_type);
+#if 0
+  /* If we get the info directly from the card, we don't have a
+     fingerprint, and parsing the subject key identifier is quite a
+     mouth full.  Let's try a different approach for now.  */
   one_attr_ext (CKA_ID, cert->fpr, 40);
+#else
+  {
+    char certptr[40];
+    snprintf (certptr, DIM (certptr), "%p", cert);
+    one_attr_ext (CKA_ID, certptr, strlen (certptr));
+  }
+#endif
 
+#if 0
+  /* For now, we disable these fields.  We can parse them from the
+     certificate just as the other data.  However, we would like to
+     avoid parsing the certificates at all, let's see how much
+     functionality we really need in the PKCS#11 token first.  */
+
+  /* This code currently only works for certificates retrieved through
+     gpgsm.  */
   if (time_to_ck_date (&cert->timestamp, &obj_start_date))
     {
       one_attr (CKA_START_DATE, obj_start_date);
     }
-  else
-    {
-      empty_attr (CKA_START_DATE);
-    }
 
   if (time_to_ck_date (&cert->expires, &obj_end_date))
     {
       one_attr (CKA_END_DATE, obj_end_date);
     }
-  else
-    {
-      empty_attr (CKA_END_DATE);
-    }
+#else
+  /* For now, we disable these fields.  We can parse them from the
+     certificate just as the other data.  However, we would like to
+     avoid parsing the certificates at all, let's see how much
+     functionality we really need in the PKCS#11 token first.  */
+  empty_attr (CKA_START_DATE);
+  empty_attr (CKA_END_DATE);
+#endif
 
   one_attr (CKA_DERIVE, obj_derive);
   one_attr (CKA_LOCAL, obj_local);
   one_attr (CKA_KEY_GEN_MECHANISM, obj_key_gen);
   one_attr (CKA_ALLOWED_MECHANISMS, obj_mechanisms);
-  
+
+#if CERT_PARSING
   one_attr_ext (CKA_SUBJECT, subject_start, subject_len);
+#endif
+
   one_attr (CKA_SENSITIVE, obj_sensitive);
   one_attr (CKA_DECRYPT, obj_decrypt);
   one_attr (CKA_SIGN, obj_sign);
@@ -599,17 +740,21 @@ scute_attr_prv (struct cert *cert, CK_ATTRIBUTE_PTR *attrp,
   empty_attr (CKA_UNWRAP_TEMPLATE);
   one_attr (CKA_ALWAYS_AUTHENTICATE, obj_always_authenticate);
 
+#if CERT_PARSING
   one_attr_ext (CKA_MODULUS, modulus_start, modulus_len);
   one_attr_ext (CKA_PUBLIC_EXPONENT, public_exp_start, public_exp_len);
+#endif
 
   if (err)
     {
+      DEBUG (DBG_INFO, "could not build private certificate object: %s",
+	     gpg_strerror (err));
       scute_attr_free (attr, attr_count);
       return err;
     }
 
   /* FIXME: Not completely safe.  */
-  assert (NR_ATTR_PRV == attr_count);
+  assert (NR_ATTR_PRV >= attr_count);
 
   *attrp = attr;
   *attr_countp = attr_count;
