@@ -1,5 +1,5 @@
 /* gpgsm.c - Talking to gpgsm.
-   Copyright (C) 2006 g10 Code GmbH
+   Copyright (C) 2006, 2008 g10 Code GmbH
 
    This file is part of Scute.
  
@@ -44,7 +44,9 @@
 #include "cryptoki.h"
 #include "support.h"
 #include "cert.h"
+#include "agent.h"
 #include "gpgsm.h"
+#include "debug.h"
 
 
 struct search
@@ -52,7 +54,7 @@ struct search
   bool found;
   cert_get_cb_t cert_get_cb;
   void *hook;
-
+  bool with_chain;
 };
 
 
@@ -83,9 +85,11 @@ search_cb (void *hook, struct cert *cert)
     }
 
   /* Add the certificate chain recursively before adding the
-     certificate.  */
-  if (strcmp (cert->chain_id, cert->fpr))
-    err = scute_gpgsm_search_certs_by_fpr (cert->chain_id, search_cb, ctx);
+     certificate.  But ignore errors.  If the chain is incomplete, we
+     might still be able to proceed, for example with client
+     authentication.  */
+  if (ctx->with_chain && strcmp (cert->chain_id, cert->fpr))
+    scute_gpgsm_search_certs_by_fpr (cert->chain_id, search_cb, ctx);
 
   /* Turn this certificate into a certificate object.  */
   err = scute_attr_cert (cert, &attrp, &attr_countp);
@@ -108,7 +112,7 @@ search_cb (void *hook, struct cert *cert)
    and ATTR_COUNTP, and for the private key object in PRV_ATTRP
    and PRV_ATTR_COUNTP.  */
 gpg_error_t
-scute_gpgsm_get_cert (char *grip, cert_get_cb_t cert_get_cb, void *hook)
+scute_gpgsm_get_cert (char *grip, int no, cert_get_cb_t cert_get_cb, void *hook)
 {
   gpg_error_t err;
   struct search search;
@@ -116,8 +120,33 @@ scute_gpgsm_get_cert (char *grip, cert_get_cb_t cert_get_cb, void *hook)
   search.found = false;
   search.cert_get_cb = cert_get_cb;
   search.hook = hook;
+  search.with_chain = false;
 
-  err = scute_gpgsm_search_certs_by_grip (grip, search_cb, &search);
+  /* If the key is from the card, we might get the certificate from
+     the card as well.  */
+  if (no >= 0)
+    {
+      struct cert cert;
+
+      memset (&cert, '\0', sizeof (cert));
+      err = scute_agent_get_cert (no, &cert);
+      if (! err)
+	{
+#if 0
+	  /* For now, we don't need no stinking chain.  */
+
+	  /* As we only have the DER certificate from the card, we need to
+	     parse that and fill out the missing info and try to get the
+	     certificate chain from gpgsm.  */
+	  err = scute_cert_from_der (&cert);
+#endif
+	  if (! err)
+	    err = search_cb (&search, &cert);
+	  return err;
+	}
+    }
   
+  search.with_chain = true;
+  err = scute_gpgsm_search_certs_by_grip (grip, search_cb, &search);
   return err;
 }
