@@ -66,7 +66,7 @@ sign_with_object (CK_SESSION_HANDLE session, CK_OBJECT_HANDLE object)
   CK_RV err;
   CK_MECHANISM mechanism = { CKM_RSA_PKCS, NULL_PTR, 0 };
   CK_BYTE data[36] = "01234567890123456789012345678901234";
-  CK_BYTE sig[128];
+  CK_BYTE sig[256];
   CK_ULONG sig_len = sizeof (sig);
 
   err = C_SignInit (session, &mechanism, object);
@@ -92,7 +92,10 @@ main (int argc, char *argv[])
   CK_RV err;
   CK_SLOT_ID_PTR slots;
   CK_ULONG slots_count;
-  int i;
+  unsigned int i;
+
+  (void) argc;
+  (void) argv;
 
   init_cryptoki ();
 
@@ -102,7 +105,7 @@ main (int argc, char *argv[])
   if (slots_count == 0)
     {
       printf ("Skipping test because no token is present.\n");
-      return 0;
+      return 77;
     }
 
   printf ("Number of slots with tokens: %lu\n", slots_count);
