@@ -40,7 +40,10 @@ main (int argc, char *argv[])
   CK_SLOT_ID_PTR slots;
   CK_SESSION_HANDLE_PTR sessions;
   CK_ULONG slots_count;
-  int i;
+  unsigned int i;
+
+  (void) argc;
+  (void) argv;
 
   init_cryptoki ();
 
@@ -50,7 +53,7 @@ main (int argc, char *argv[])
   if (slots_count == 0)
     {
       printf ("Skipping test because no token is present.\n");
-      return 0;
+      return 77;
     }
 
   printf ("Number of slots with tokens: %lu\n", slots_count);
