@@ -29,6 +29,7 @@
 
 #include <stdio.h>
 #include <stdbool.h>
+#include <string.h>
 #include <ctype.h>
 
 #include "t-support.h"
@@ -37,12 +38,12 @@
 bool printable;
 
 CK_RV
-dump_one (CK_ATTRIBUTE_PTR attr, unsigned char *data, int max_size)
+dump_one (CK_ATTRIBUTE_PTR attr, unsigned char *data, unsigned int max_size)
 {
-  int i;
+  unsigned int i;
   int col;
 
-  if (attr->ulValueLen < 0 || attr->ulValueLen > max_size)
+  if (attr->ulValueLen > max_size)
     return CKR_GENERAL_ERROR;
 
   col = 0;
@@ -227,15 +228,18 @@ dump_object (CK_SESSION_HANDLE session, CK_OBJECT_HANDLE object)
 	printf ("     Certificate Check Value: %02x%02x%02x\n",
 		cert_check[0], cert_check[1], cert_check[2]);
 
-	fail_if_err ((cert_attr[8].ulValueLen != sizeof (cert_sdate)) ?
-		     CKR_GENERAL_ERROR : 0);
-	printf ("     Certificate Start Date: %.4s/%.2s/%.2s\n",
-		cert_sdate.year, cert_sdate.month, cert_sdate.day);
+	if (cert_attr[8].ulValueLen && cert_attr[9].ulValueLen)
+	  {
+	    fail_if_err ((cert_attr[8].ulValueLen != sizeof (cert_sdate)) ?
+			 CKR_GENERAL_ERROR : 0);
+	    printf ("     Certificate Start Date: %.4s/%.2s/%.2s\n",
+		    cert_sdate.year, cert_sdate.month, cert_sdate.day);
 
-	fail_if_err ((cert_attr[9].ulValueLen != sizeof (cert_edate)) ?
-		     CKR_GENERAL_ERROR : 0);
-	printf ("     Certificate End Date: %.4s/%.2s/%.2s\n",
-		cert_edate.year, cert_edate.month, cert_edate.day);
+	    fail_if_err ((cert_attr[9].ulValueLen != sizeof (cert_edate)) ?
+			 CKR_GENERAL_ERROR : 0);
+	    printf ("     Certificate End Date: %.4s/%.2s/%.2s\n",
+		    cert_edate.year, cert_edate.month, cert_edate.day);
+	  }
 
 	printf ("     Certificate Subject: Length %lu\n",
 		cert_attr[10].ulValueLen);
@@ -411,15 +415,18 @@ dump_object (CK_SESSION_HANDLE session, CK_OBJECT_HANDLE object)
 	err = dump_one (&key_attr[5], key_id, sizeof (key_id));
 	fail_if_err (err);
 
-	fail_if_err ((key_attr[6].ulValueLen != sizeof (key_sdate)) ?
-		     CKR_GENERAL_ERROR : 0);
-	printf ("     Key Start Date: %.4s/%.2s/%.2s\n",
-		key_sdate.year, key_sdate.month, key_sdate.day);
-
-	fail_if_err ((key_attr[7].ulValueLen != sizeof (key_edate)) ?
-		     CKR_GENERAL_ERROR : 0);
-	printf ("     Key End Date: %.4s/%.2s/%.2s\n",
-		key_edate.year, key_edate.month, key_edate.day);
+	if (key_attr[6].ulValueLen && key_attr[7].ulValueLen)
+	  {
+	    fail_if_err ((key_attr[6].ulValueLen != sizeof (key_sdate)) ?
+			 CKR_GENERAL_ERROR : 0);
+	    printf ("     Key Start Date: %.4s/%.2s/%.2s\n",
+		    key_sdate.year, key_sdate.month, key_sdate.day);
+	    
+	    fail_if_err ((key_attr[7].ulValueLen != sizeof (key_edate)) ?
+			 CKR_GENERAL_ERROR : 0);
+	    printf ("     Key End Date: %.4s/%.2s/%.2s\n",
+		    key_edate.year, key_edate.month, key_edate.day);
+	  }
 
 	fail_if_err ((key_attr[8].ulValueLen != sizeof (key_derive)) ?
 		     CKR_GENERAL_ERROR : 0);
@@ -526,7 +533,10 @@ main (int argc, char *argv[])
   CK_RV err;
   CK_SLOT_ID_PTR slots;
   CK_ULONG slots_count;
-  int i;
+  unsigned int i;
+
+  (void) argc;
+  (void) argv;
 
   if (argc > 1 && !strcmp ("--printable", argv[1]))
     printable = true;
@@ -539,7 +549,7 @@ main (int argc, char *argv[])
   if (slots_count == 0)
     {
       printf ("Skipping test because no token is present.\n");
-      return 0;
+      return 77;
     }
 
   printf ("Number of slots with tokens: %lu\n", slots_count);
